@@ -30,13 +30,17 @@
 @file      simd.h
 @brief     RE/flex SIMD intrinsics
 @author    Robert van Engelen - engelen@genivia.com
-@copyright (c) 2016-2022, Robert van Engelen, Genivia Inc. All rights reserved.
+@copyright (c) 2016-2024, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
 */
 
 #ifndef SIMD_H
 #define SIMD_H
 
+#include <cstddef>
+#include <cstdint>
+#include <cstdlib>
+
 #if defined(HAVE_AVX512BW)
 # include <immintrin.h>
 #elif defined(HAVE_AVX2)
@@ -133,15 +137,22 @@ inline uint32_t popcountl(uint64_t x)
 }
 #endif
 
-// Partially count newlines in string b up to and including position e in b, updates b close to e with uncounted part
+// Partially count newlines in string b up to e, updates b close to e with uncounted part
 extern size_t simd_nlcount_sse2(const char*& b, const char *e);
 extern size_t simd_nlcount_avx2(const char*& b, const char *e);
 extern size_t simd_nlcount_avx512bw(const char*& b, const char *e);
 
 } // namespace reflex
 
-#endif
+#elif defined(HAVE_NEON)
+
+namespace reflex {
+
+// Partially count newlines in string b up to e, updates b close to e with uncounted part
+extern size_t simd_nlcount_neon(const char*& b, const char *e);
+
+} // namespace reflex
 
 #endif
 
-
+#endif
